// Скрипт подписывается на топики ошибок связи и выводит в UI информацию о ошибках
var Device      = "Check_serial_error";
var NameEN      = "Check serial error";
var NameRU      = "Обработка ошибок связи"
var nDevice     = Device;
var sErrorTXT   = "/Error_TXT";
var sWarningTXT = "/Warning_TXT";
var sResetError = "/pbReset";

//----------------СОЗДАНИЕ ВИРТУАЛЬНОГО УСТРОЙСТВА----------------------//
// Функция создание виртуального устройства 
function Create_VDevice(vdDevice,vdNameEN,vdNameRU) {
    try {
        return defineVirtualDevice(vdDevice, {
            // имя виджета в GUI
            title: { en: vdNameEN, ru: vdNameRU},
            cells: {
                // Текущая ошибка WEB UI
                Error_TXT: {
                    title: { en: "Current Error", ru: "Текущая ошибка" },
                    type: "text",
                    value: "Отсутствует",
                    readonly: true,
                    forceDefault: true,
                    order: 1,
                },
                  // Текущая предупреждение WEB UI
                Warning_TXT: {
                    title: { en: "Current warning", ru: "Предупреждение" },
                    type: "text",
                    value: "Отсутствует",
                    readonly: true,
                    forceDefault: true,
                    order: 2,
                },
                // Кнопка сброса ошибки.
                pbReset: {
                    title: { en: "Reset error", ru: "Сброс ошибки" },
                    type: "pushbutton",
                    value: "false",
                    forceDefault: true,
                    order: 3,
                },
            },
        });
    } catch (error) {
        log.error("try Error Create " + NameVD + "  =" + error);
        return undefined;
    };
};

// Функция подписки на изменение топика кнопки "СБРОС ОШИБКИ" в WEB UI
function RuleResetError(RuleName) {
    try {
        return defineRule(RuleName, {
            whenChanged: Device + sResetError,                    // подписка на событие нажатия на кнопку "Сброс ошибки" в WEB UI
            then: function (newValue, devName, cellName) {        // devName = Control cellName = pbResetFID
                log("Произведен ручной сброс ошибки");            // логирование в консоли
                //ResetCurrentError();                            // Вызываем функцию сброса ошибок
                dev[Device + sWarningTXT] = "Отсутствует";        // Очищаем поле предупреждений
                dev[nDevice + sErrorTXT] = "Отсутствует";         // Выводим подсказку  в WEB UI
            },
        });
    } catch (error) {
        log.error("try Error RuleResetError: '" + error + "'");
        return undefined;
    };
};

//----------------ФУНКЦИИ СОЗДАНИЯ ПОДПИСОК НА ТОПИКИ MQTT----------------------//
// Функция создания правила подписки на пропадании связи с устройствами 
function CheckSerialDevices (){
    try {
        // Подписка на все устройства в которых есть топики /meta/error"
        return trackMqtt("/devices/+/controls/+/meta/error", function(message){     // Подписка на все устройства в которых есть топики /meta/error"
        // log.info("name: {}, value: {}".format(message.topic, message.value))
            
            // Функция извлечения текста в виде "устройство/топик"
            function GetDeviceName(string) {
                var parts = string.split('/');                                      // Разделяем строку по символу '/'
                return parts[2] + '/' + parts[4];                                   // Извлекаем нужные части и объединяем их
            };

            // Преобразовываем полученное значение для дальнейшего использования
            var ShortDeviceString = GetDeviceName(message.topic); 

            // В зависимости от полученного значения ошибки (message.value) выполняем различные действия  
            switch (message.value) {

                // Если текущая ошибка равна "r" нет ответа от устройства то
                case "r":
                    dev[nDevice + sErrorTXT] = "Ошибка чтения из устройства";  // Выводим подсказку  в WEB UI
                    dev[nDevice + sWarningTXT] = ShortDeviceString;            // Выводим подсказку  в WEB UI
                    log.error(
                        "Ошибка чтения из устройства:  {}",
                        ShortDeviceString
                    );
                    break;
                    
                // Если текущая ошибка равна "w" запись в устройство не удалась 
                case "w":
                    dev[nDevice + sErrorTXT] = "Ошибка записи в устройство";   // Выводим подсказку  в WEB UI
                    dev[nDevice + sWarningTXT] = ShortDeviceString;            // Выводим подсказку  в WEB UI
                    log.error(
                        "Ошибка записи в устройств  {}",
                        ShortDeviceString
                    );
                    break;

                // Если текущая ошибка равна "rp" попытка возобновить чтение после таймаута не удалась 
                case "rp":
                    dev[nDevice + sErrorTXT] = "Ошибка возобновления чтения";  // Выводим подсказку  в WEB UI
                    dev[nDevice + sWarningTXT] = ShortDeviceString;            // Выводим подсказку  в WEB UI
                    log.error(                                                 // Выводим в лог
                        "Попытка возобновить чтение с устройства:  {} не удалась",
                        ShortDeviceString
                    );
                    break;

                // Если текущая ошибка равна "wp". Прим. (сюда не разу не попал наверно такого не бывает)      
                case "wp":
                    log.warning(
                        "Попытка возобновить запись в устройство?:  {}, ",        // Выводим в лог
                        ShortDeviceString
                    );
                break;

                // Если текущая ошибка равна (p)периодическая попытка чтения? 
                case "p":
                    dev[nDevice + sErrorTXT] = "Ошибка возобновления";         // Выводим подсказку  в WEB UI
                    dev[nDevice + sWarningTXT] = ShortDeviceString;            // Выводим подсказку  в WEB UI
                    log.error(                                                 // Выводим в лог
                        "Попытка возобновить чтение с устройства:  {} не удалась",
                        ShortDeviceString
                    );
                    break;

                // Насколько понял сюда попадем в случае возобновления связи с устройством (чтение или запись которые раньше не читались или не записывались)
                default:
                    dev[nDevice + sErrorTXT] = "Связь восстановлена";          // Выводим подсказку  в WEB UI
                    dev[nDevice + sWarningTXT] = ShortDeviceString;            // Выводим подсказку  в WEB UI
                    log.warning(
                        "Возобновление связи c устройством:  {}, код: ({})",      // Выводим в лог
                        ShortDeviceString, message.value
                    );
                    break;
            };
      });
    // При исключении выводим в MQTT и лог значение которое привело  к этому        
    } catch (error) {
        dev[nDevice + sErrorTXT] = " Try checkSerialDevices: " + error;
        log.error("Try checkSerialDevices: " + error);
        return null;  
    };
};
//----------------НЕПОСРЕДСТВЕННОЕ СОЗДАНИЕ ----------------------//
var vdCreateCheckSerialError = Create_VDevice (Device,NameEN,NameRU);
var ChangedCheck = CheckSerialDevices ();
var ChangedButton = RuleResetError ("RuleResetError");